﻿//-----------------------------------------------------------------------
// <copyright file="GuessGame.cs" company="BME MIT" />
//-----------------------------------------------------------------------
/*
 * Name:    IRF.ConsoleGuessGame.GuessGame
 * Author:  Zoltan Micskei, BME MIT
 * Date:    2011.04.17.
 * Desc:    ConsoleGuessGame sample application for demonstrating manageable applications 
 *          see: http://www.inf.mit.bme.hu/edu/bsc/irf
 *          
 *          "Business logic", implements a game, where in each round a random number is chosen, and the user
 *          makes guesses, and the game returns whether it is lower/higher/equal than the chosen number
 *          
 * Param:   name of the game session
 */

namespace IRF.ConsoleGuessGame
{
    using System;
    using System.Diagnostics;

    /// <summary>
    /// Possible results of a guess
    /// </summary>
    public enum GuessResult
    {
        /// <summary>
        /// The guess is lower than the secret number.
        /// </summary>
        TooLow,

        /// <summary>
        /// The guess is higher than the secret number.
        /// </summary>
        TooHigh,

        /// <summary>
        /// The guess equals the secret number.
        /// </summary>
        Success,

        /// <summary>
        /// The guess is not in the range of accepted numbers.
        /// </summary>
        WrongInput
    }

    /// <summary>
    /// Class implementing the number guessing game
    /// </summary>
    internal class GuessGame
    {        
        /// <summary>
        /// The secret number to guess chosen in each round.
        /// </summary>
        private int secretNumber;
        
        /// <summary>
        /// Pseudo-random generator for chosing the secret number in each round
        /// </summary>
        private Random random = new Random();
        
        /// <summary>
        /// Object used for locking when accessing the shared fields of the class
        /// </summary>
        private object lockObject = new object();

        /// <summary>
        /// TotalGuesses performance counter
        /// </summary>
        private PerformanceCounter totalGuessesCounter;
        
        /// <summary>
        /// TotalFaultyGuesses performance counter
        /// </summary>
        private PerformanceCounter totalFaultyGuessesCounter;

        /// <summary>
        /// Initializes a new instance of the GuessGame class. Starts a new round.  
        /// </summary>
        /// <param name="sessionName">name of the game session</param>
        public GuessGame(string sessionName)
        {
            this.SessionName = sessionName;            

            // connect to perforance counters in write mode
            this.totalGuessesCounter = new PerformanceCounter(Counters.CategoryName, "TotalGuesses", this.SessionName, false);
            this.totalFaultyGuessesCounter = new PerformanceCounter(Counters.CategoryName, "TotalFaultyGuesses", this.SessionName, false);

            this.NewRound();
        }

        /// <summary>
        /// Gets Name of the game session supplied in the startup parameters of the application
        /// </summary>
        public string SessionName { get; private set; }        

        // TODO TotalGuesses should also use locking
       
        // TODO this property is a bit redudant as the totalGuessesCounter contains this data,
        //  just keeping WMI properties and perf counters separate for demonstrational purposes

        /// <summary>
        /// Gets the TotalGuesses metric
        /// </summary>        
        public int TotalGuesses { get; private set; }

        // TODO TotalFaultyGuesses should also use locking
        
        /// <summary>
        /// Gets the TotalFaultyGuesses metric
        /// </summary>
        public int TotalFaultyGuesses { get; private set; }        

        /// <summary>
        /// Chooses a new secret number and resets the metrics.
        /// </summary>
        public void NewRound()
        {
            lock (this.lockObject)
            {
                this.secretNumber = this.random.Next(100);
                this.TotalGuesses = 0;
                this.TotalFaultyGuesses = 0;

                // reset performance counters also
                this.totalGuessesCounter.RawValue = 0;
                this.totalFaultyGuessesCounter.RawValue = 0;                
            }            
        }

        /// <summary>
        /// Accepts a new guess.
        /// </summary>
        /// <param name="guess">A guess between 0-100</param>
        /// <returns>The result of the current guess</returns>
        public GuessResult Guess(int guess)
        {
            GuessResult result = GuessResult.WrongInput;
            this.TotalGuesses++;
            this.totalGuessesCounter.Increment();

            if (guess < 0 || guess > 100)
            {
                this.TotalFaultyGuesses++;
                this.totalFaultyGuessesCounter.Increment();

                result = GuessResult.WrongInput;
            }
            else
            {                
                if (guess < this.secretNumber)
                {
                    result = GuessResult.TooLow;
                }

                if (guess > this.secretNumber)
                {
                    result = GuessResult.TooHigh;
                }

                if (guess == this.secretNumber)
                {
                    result = GuessResult.Success;
                }
            }

            return result;
        }
    }
}
